@@ -2,14 +2,16 @@
 #include "ControlSocket.h"
 #include "directorycache.h"
 #include "engineprivate.h"
-#include "event_loop.h"
-#include "local_filesys.h"
 #include "local_path.h"
 #include "logging_private.h"
 #include "proxy.h"
 #include "servercapabilities.h"
 #include "sizeformatting_base.h"
 
+#include <libfilezilla/event_loop.hpp>
+#include <libfilezilla/iputils.hpp>
+#include <libfilezilla/local_filesys.hpp>
+
 #include <wx/file.h>
 #include <wx/filename.h>
 
@@ -26,7 +28,7 @@
 #endif
 
 struct obtain_lock_event_type;
-typedef CEvent<obtain_lock_event_type> CObtainLockEvent;
+typedef fz::simple_event<obtain_lock_event_type> CObtainLockEvent;
 
 std::list<CControlSocket::t_lockInfo> CControlSocket::m_lockInfoList;
 
@@ -49,7 +51,7 @@ COpData::~COpData()
 
 CControlSocket::CControlSocket(CFileZillaEnginePrivate & engine)
 	: CLogging(engine)
-	, CEventHandler(engine.event_loop_)
+	, event_handler(engine.event_loop_)
 	, engine_(engine)
 {
 	m_pCurOpData = 0;
@@ -66,7 +68,7 @@ CControlSocket::CControlSocket(CFileZillaEnginePrivate & engine)
 
 CControlSocket::~CControlSocket()
 {
-	RemoveHandler();
+	remove_handler();
 
 	DoClose();
 
@@ -96,7 +98,7 @@ void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpDat
 
 	CTransferStatus const status = engine_.transfer_status_.Get(tmp);
 	if (!status.empty() && (nErrorCode == FZ_REPLY_OK || status.madeProgress)) {
-		int elapsed = static_cast<int>((CDateTime::Now() - status.started).get_seconds());
+		int elapsed = static_cast<int>((fz::datetime::now() - status.started).get_seconds());
 		if (elapsed <= 0)
 			elapsed = 1;
 		wxString time = wxString::Format(
@@ -256,15 +258,16 @@ int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 wxString CControlSocket::ConvertDomainName(wxString const& domain)
 {
 #ifdef __WXMSW__
-	int len = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain, domain.size() + 1, 0, 0);
-	if( !len ) {
+	int len = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain.wc_str(), domain.size() + 1, 0, 0);
+	if (!len) {
 		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
 		return domain;
 	}
 
 	wchar_t* output = new wchar_t[len];
-	int res = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain, domain.size() + 1, output, len);
-	if( !res ) {
+	int res = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain.wc_str(), domain.size() + 1, output, len);
+	if (!res) {
+		delete [] output;
 		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
 		return domain;
 	}
@@ -386,7 +389,7 @@ int CControlSocket::CheckOverwriteFile()
 		found = false;
 
 	if (!pData->download) {
-		if (!found && pData->remoteFileSize < 0 && !pData->fileTime.IsValid())
+		if (!found && pData->remoteFileSize < 0 && !pData->fileTime.empty())
 			return FZ_REPLY_OK;
 	}
 
@@ -407,13 +410,13 @@ int CControlSocket::CheckOverwriteFile()
 	else
 		pNotification->canResume = false;
 
-	pNotification->localTime = CLocalFileSystem::GetModificationTime(pData->localFile);
+	pNotification->localTime = fz::local_filesys::get_modification_time(fz::to_native(pData->localFile));
 
-	if (pData->fileTime.IsValid())
+	if (pData->fileTime.empty())
 		pNotification->remoteTime = pData->fileTime;
 
 	if (found) {
-		if (!pData->fileTime.IsValid()) {
+		if (!pData->fileTime.empty()) {
 			if (entry.has_date()) {
 				pNotification->remoteTime = entry.time;
 				pData->fileTime = entry.time;
@@ -544,32 +547,32 @@ wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8 /
 	return buffer;
 }
 
-void CControlSocket::OnTimer(timer_id)
+void CControlSocket::OnTimer(fz::timer_id)
 {
 	m_timer = 0; // It's a one-shot timer, no need to stop it
 
 	int const timeout = engine_.GetOptions().GetOptionVal(OPTION_TIMEOUT);
 	if (timeout > 0) {
-		duration elapsed = CMonotonicClock::now() - m_lastActivity;
+		fz::duration elapsed = fz::monotonic_clock::now() - m_lastActivity;
 
 		if ((!m_pCurOpData || !m_pCurOpData->waitForAsyncRequest) && !IsWaitingForLock()) {
-			if (elapsed > duration::from_seconds(timeout)) {
+			if (elapsed > fz::duration::from_seconds(timeout)) {
 				LogMessage(MessageType::Error, wxPLURAL("Connection timed out after %d second of inactivity", "Connection timed out after %d seconds of inactivity", timeout), timeout);
 				DoClose(FZ_REPLY_TIMEOUT);
 				return;
 			}
 		}
 		else {
-			elapsed = duration();
+			elapsed = fz::duration();
 		}
 
-		m_timer = AddTimer(duration::from_milliseconds(timeout * 1000) - elapsed, true);
+		m_timer = add_timer(fz::duration::from_milliseconds(timeout * 1000) - elapsed, true);
 	}
 }
 
 void CControlSocket::SetAlive()
 {
-	m_lastActivity = CMonotonicClock::now();
+	m_lastActivity = fz::monotonic_clock::now();
 }
 
 void CControlSocket::SetWait(bool wait)
@@ -578,16 +581,16 @@ void CControlSocket::SetWait(bool wait)
 		if (m_timer)
 			return;
 
-		m_lastActivity = CMonotonicClock::now();
+		m_lastActivity = fz::monotonic_clock::now();
 
 		int timeout = engine_.GetOptions().GetOptionVal(OPTION_TIMEOUT);
 		if (!timeout)
 			return;
 
-		m_timer = AddTimer(duration::from_milliseconds(timeout * 1000 + 100), true); // Add a bit of slack
+		m_timer = add_timer(fz::duration::from_milliseconds(timeout * 1000 + 100), true); // Add a bit of slack
 	}
 	else {
-		StopTimer(m_timer);
+		stop_timer(m_timer);
 		m_timer = 0;
 	}
 }
@@ -739,7 +742,7 @@ void CControlSocket::UnlockCache()
 			continue;
 
 		// Send notification
-		lockInfo.pControlSocket->SendEvent<CObtainLockEvent>();
+		lockInfo.pControlSocket->send_event<CObtainLockEvent>();
 		break;
 	}
 }
@@ -811,13 +814,13 @@ void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
 	}
 }
 
-duration CControlSocket::GetTimezoneOffset() const
+fz::duration CControlSocket::GetTimezoneOffset() const
 {
-	duration ret;
+	fz::duration ret;
 	if (m_pCurrentServer) {
 		int seconds = 0;
 		if (CServerCapabilities::GetCapability(*m_pCurrentServer, timezone_offset, &seconds) == yes) {
-			ret = duration::from_seconds(seconds);
+			ret = fz::duration::from_seconds(seconds);
 		}
 	}
 	return ret;
@@ -898,9 +901,9 @@ bool CRealControlSocket::Send(const char *buffer, int len)
 	return true;
 }
 
-void CRealControlSocket::operator()(CEventBase const& ev)
+void CRealControlSocket::operator()(fz::event_base const& ev)
 {
-	if (!Dispatch<CSocketEvent, CHostAddressEvent>(ev, this,
+	if (!fz::dispatch<CSocketEvent, CHostAddressEvent>(ev, this,
 		&CRealControlSocket::OnSocketEvent,
 		&CRealControlSocket::OnHostAddress))
 	{
@@ -918,6 +921,7 @@ void CRealControlSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, i
 	case SocketEventType::connection_next:
 		if (error)
 			LogMessage(MessageType::Status, _("Connection attempt failed with \"%s\", trying next address."), CSocket::GetErrorDescription(error));
+		SetAlive();
 		break;
 	case SocketEventType::connection:
 		if (error) {
@@ -947,7 +951,7 @@ void CRealControlSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, i
 	}
 }
 
-void CRealControlSocket::OnHostAddress(CSocketEventSource*, wxString const& address)
+void CRealControlSocket::OnHostAddress(CSocketEventSource*, std::string const& address)
 {
 	if (!m_pBackend)
 		return;
@@ -1074,10 +1078,11 @@ int CRealControlSocket::ContinueConnect()
 			port = m_pCurrentServer->GetPort();
 		}
 	}
-	if (!IsIpAddress(host))
+	if (fz::get_address_type(host.ToStdWstring()) == fz::address_type::unknown) {
 		LogMessage(MessageType::Status, _("Resolving address of %s"), host);
+	}
 
-	int res = m_pSocket->Connect(host, port);
+	int res = m_pSocket->Connect(fz::to_native(host), port);
 
 	// Treat success same as EINPROGRESS, we wait for connect notification in any case
 	if (res && res != EINPROGRESS) {
@@ -1135,11 +1140,11 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		SendNextCommand();
 		break;
 	case CFileExistsNotification::overwriteNewer:
-		if (!pFileExistsNotification->localTime.IsValid() || !pFileExistsNotification->remoteTime.IsValid())
+		if (!pFileExistsNotification->localTime.empty() || !pFileExistsNotification->remoteTime.empty())
 			SendNextCommand();
-		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.IsEarlierThan(pFileExistsNotification->remoteTime))
+		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.earlier_than(pFileExistsNotification->remoteTime))
 			SendNextCommand();
-		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.IsLaterThan(pFileExistsNotification->remoteTime))
+		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.later_than(pFileExistsNotification->remoteTime))
 			SendNextCommand();
 		else
 		{
@@ -1172,15 +1177,15 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		}
 		break;
 	case CFileExistsNotification::overwriteSizeOrNewer:
-		if (!pFileExistsNotification->localTime.IsValid() || !pFileExistsNotification->remoteTime.IsValid())
+		if (!pFileExistsNotification->localTime.empty() || !pFileExistsNotification->remoteTime.empty())
 			SendNextCommand();
 		/* First compare flags both size known but different, one size known and the other not (obviously they are different).
 		Second compare flags the remaining case in which we need to send command : both size unknown */
 		else if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize < 0))
 			SendNextCommand();
-		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.IsEarlierThan(pFileExistsNotification->remoteTime))
+		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.earlier_than(pFileExistsNotification->remoteTime))
 			SendNextCommand();
-		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.IsLaterThan(pFileExistsNotification->remoteTime))
+		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.later_than(pFileExistsNotification->remoteTime))
 			SendNextCommand();
 		else
 		{
@@ -1211,7 +1216,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 
 			int64_t size;
 			bool isLink;
-			if (CLocalFileSystem::GetFileInfo(pData->localFile, isLink, &size, 0, 0) == CLocalFileSystem::file)
+			if (fz::local_filesys::get_file_info(fz::to_native(pData->localFile), isLink, &size, 0, 0) == fz::local_filesys::file)
 				pData->localFileSize = size;
 			else
 				pData->localFileSize = -1;
@@ -1236,7 +1241,7 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 					break;
 			}
 			else {
-				pData->fileTime = CDateTime();
+				pData->fileTime = fz::datetime();
 				pData->remoteFileSize = -1;
 			}
 
@@ -1285,7 +1290,7 @@ void CControlSocket::CreateLocalDir(const wxString &local_file)
 		local_path.AddSegment(*iter);
 
 #ifdef __WXMSW__
-		BOOL res = CreateDirectory(local_path.GetPath(), 0);
+		BOOL res = CreateDirectory(local_path.GetPath().wc_str(), 0);
 		if (!res && GetLastError() != ERROR_ALREADY_EXISTS)
 			break;
 #else
@@ -1349,12 +1354,11 @@ int CControlSocket::Chmod(const CChmodCommand&)
 	return FZ_REPLY_NOTSUPPORTED;
 }
 
-void CControlSocket::operator()(CEventBase const& ev)
+void CControlSocket::operator()(fz::event_base const& ev)
 {
-	if (Dispatch<CTimerEvent>(ev, this, &CControlSocket::OnTimer)) {
-		return;
-	}
-	Dispatch<CObtainLockEvent>(ev, this, &CControlSocket::OnObtainLock);
+	fz::dispatch<fz::timer_event, CObtainLockEvent>(ev, this,
+		&CControlSocket::OnTimer,
+		&CControlSocket::OnObtainLock);
 }
 
 void CControlSocket::SetActive(CFileZillaEngine::_direction direction)
