@@ -9,8 +9,10 @@
 #include "proxy.h"
 #include "servercapabilities.h"
 
+#include <libfilezilla/util.hpp>
+
 CTransferSocket::CTransferSocket(CFileZillaEnginePrivate & engine, CFtpControlSocket & controlSocket, TransferMode transferMode)
-: CEventHandler(controlSocket.event_loop_)
+: fz::event_handler(controlSocket.event_loop_)
 , engine_(engine)
 , controlSocket_(controlSocket)
 , m_transferMode(transferMode)
@@ -19,7 +21,7 @@ CTransferSocket::CTransferSocket(CFileZillaEnginePrivate & engine, CFtpControlSo
 
 CTransferSocket::~CTransferSocket()
 {
-	RemoveHandler();
+	remove_handler();
 	if (m_transferEndReason == TransferEndReason::none)
 		m_transferEndReason = TransferEndReason::successful;
 	ResetSocket();
@@ -51,7 +53,7 @@ void CTransferSocket::ResetSocket()
 
 }
 
-wxString CTransferSocket::SetupActiveTransfer(const wxString& ip)
+wxString CTransferSocket::SetupActiveTransfer(std::string const& ip)
 {
 	ResetSocket();
 	m_pSocketServer = CreateSocketServer();
@@ -70,6 +72,14 @@ wxString CTransferSocket::SetupActiveTransfer(const wxString& ip)
 		return wxString();
 	}
 
+	if (engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS)) {
+		port += static_cast<int>(engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS_OFFSET));
+		if (port <= 0 || port >= 65536) {
+			controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Port outside valid range"));
+			return wxString();
+		}
+	}
+
 	wxString portArguments;
 	if (m_pSocketServer->GetAddressFamily() == CSocket::ipv6) {
 		portArguments = wxString::Format(_T("|2|%s|%d|"), ip, port);
@@ -83,7 +93,7 @@ wxString CTransferSocket::SetupActiveTransfer(const wxString& ip)
 	return portArguments;
 }
 
-void CTransferSocket::OnSocketEvent(CSocketEventSource* source, SocketEventType t, int error)
+void CTransferSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
 {
 	if (m_pProxyBackend)
 	{
@@ -302,7 +312,7 @@ void CTransferSocket::OnReceive()
 			FinalizeWrite();
 		}
 		else {
-			SendEvent<CSocketEvent>(m_pBackend, SocketEventType::read, 0);
+			send_event<CSocketEvent>(m_pBackend, SocketEventType::read, 0);
 		}
 	}
 	else if (m_transferMode == TransferMode::resumetest) {
@@ -403,7 +413,7 @@ void CTransferSocket::OnSend()
 		}
 	}
 	else if (written > 0) {
-		SendEvent<CSocketEvent>(m_pBackend, SocketEventType::write, 0);
+		send_event<CSocketEvent>(m_pBackend, SocketEventType::write, 0);
 	}
 }
 
@@ -489,15 +499,14 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 											 host, port,
 											 controlSocket_.m_pProxyBackend->GetUser(), controlSocket_.m_pProxyBackend->GetPass());
 
-		if (res != EINPROGRESS)
-		{
+		if (res != EINPROGRESS) {
 			ResetSocket();
 			return false;
 		}
 		int error;
 		host = controlSocket_.m_pSocket->GetPeerIP();
 		port = controlSocket_.m_pSocket->GetRemotePort(error);
-		if( host.empty() || port < 1 ) {
+		if (host.empty() || port < 1) {
 			controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Could not get peer address of control connection."));
 			ResetSocket();
 			return false;
@@ -506,7 +515,30 @@ bool CTransferSocket::SetupPassiveTransfer(wxString host, int port)
 
 	SetSocketBufferSizes(m_pSocket);
 
-	int res = m_pSocket->Connect(host, port);
+	// Try to bind the source IP of the data connection to the same IP as the control connection.
+	// We can do so either if 
+	// 1) the destination IP of the data connection matches peer IP of the control connection or
+	// 2) we are using a proxy.
+	//
+	// In case destination IPs of control and data connection are different, do not bind to the
+	// same source.
+
+	std::string bindAddress;
+	if (m_pProxyBackend) {
+		bindAddress = controlSocket_.m_pSocket->GetLocalIP();
+		controlSocket_.LogMessage(MessageType::Debug_Info, _T("Binding data connection source IP to control connection source IP %s"), bindAddress);
+	}
+	else {
+		if (controlSocket_.m_pSocket->GetPeerIP(true) == host || controlSocket_.m_pSocket->GetPeerIP(false) == host) {
+			bindAddress = controlSocket_.m_pSocket->GetLocalIP();
+			controlSocket_.LogMessage(MessageType::Debug_Info, _T("Binding data connection source IP to control connection source IP %s"), bindAddress);
+		}
+		else {
+			controlSocket_.LogMessage(MessageType::Debug_Warning, _T("Destination IP of data connection does not match peer IP of control connection. Not binding source address of data connection."));
+		}
+	}
+
+	int res = m_pSocket->Connect(fz::to_native(host), port, CSocket::unspec, bindAddress);
 	if (res && res != EINPROGRESS) {
 		ResetSocket();
 		return false;
@@ -543,7 +575,7 @@ void CTransferSocket::TransferEnd(TransferEndReason reason)
 
 	ResetSocket();
 
-	engine_.SendEvent<CFileZillaEngineEvent>(engineTransferEnd);
+	engine_.send_event<CFileZillaEngineEvent>(engineTransferEnd);
 }
 
 CSocket* CTransferSocket::CreateSocketServer(int port)
@@ -563,8 +595,7 @@ CSocket* CTransferSocket::CreateSocketServer(int port)
 
 CSocket* CTransferSocket::CreateSocketServer()
 {
-	if (!engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS))
-	{
+	if (!engine_.GetOptions().GetOptionVal(OPTION_LIMITPORTS)) {
 		// Ask the systen for a port
 		CSocket* pServer = CreateSocketServer(0);
 		return pServer;
@@ -588,7 +619,7 @@ CSocket* CTransferSocket::CreateSocketServer()
 		low = high;
 
 	if (start < low || start > high) {
-		start = GetRandomNumber(low, high);
+		start = fz::random_number(low, high);
 		wxASSERT(start >= low && start <= high);
 	}
 
@@ -674,6 +705,8 @@ void CTransferSocket::OnIOThreadEvent()
 void CTransferSocket::FinalizeWrite()
 {
 	bool res = ioThread_->Finalize(BUFFERSIZE - m_transferBufferLen);
+	m_transferBufferLen = BUFFERSIZE;
+
 	if (m_transferEndReason != TransferEndReason::none)
 		return;
 
@@ -695,7 +728,7 @@ bool CTransferSocket::InitTls(const CTlsSocket* pPrimaryTlsSocket)
 	m_pSocket->SetFlags(m_pSocket->GetFlags() | CSocket::flag_nodelay);
 
 	wxASSERT(!m_pBackend);
-	m_pTlsSocket = new CTlsSocket(this, m_pSocket, &controlSocket_);
+	m_pTlsSocket = new CTlsSocket(this, *m_pSocket, &controlSocket_);
 
 	if (!m_pTlsSocket->Init()) {
 		delete m_pTlsSocket;
@@ -764,9 +797,9 @@ void CTransferSocket::SetSocketBufferSizes(CSocket* pSocket)
 	pSocket->SetBufferSizes(size_read, size_write);
 }
 
-void CTransferSocket::operator()(CEventBase const& ev)
+void CTransferSocket::operator()(fz::event_base const& ev)
 {
-	Dispatch<CSocketEvent, CIOThreadEvent>(ev, this,
+	fz::dispatch<CSocketEvent, CIOThreadEvent>(ev, this,
 		&CTransferSocket::OnSocketEvent,
 		&CTransferSocket::OnIOThreadEvent);
 }
