@@ -15,9 +15,9 @@ class CLocalFileData final : public CGenericFileData
 public:
 	wxString name;
 #ifdef __WXMSW__
-	CSparseOptional<wxString> label;
+	fz::sparse_optional<wxString> label;
 #endif
-	CDateTime time;
+	fz::datetime time;
 	int64_t size;
 	int attributes;
 	bool dir;
@@ -59,9 +59,9 @@ protected:
 #endif
 
 public:
-	virtual bool CanStartComparison(wxString* pError);
+	virtual bool CanStartComparison();
 	virtual void StartComparison();
-	virtual bool GetNextFile(wxString& name, bool &dir, wxLongLong &size, CDateTime& date);
+	virtual bool get_next_file(wxString& name, bool &dir, int64_t &size, fz::datetime& date);
 	virtual void FinishComparison();
 
 	virtual bool ItemIsDir(int index) const;
